set.seed(12345)
N <- 1000  #-Dimension of the covariance matrix.

eigenvalues <- seq(10, 1, length.out = N)  
L <- diag(eigenvalues)
RandMat <- matrix(rnorm(N^2), nrow = N)
Q <- qr.Q(qr(RandMat))
M <- Q %*% L %*% t(Q)
M <<- (M + t(M))/2  # fix round-off errors


ObjFunMax <- function(x){
  ids <- order(x)
  idx <- ids[1:k]
  A <- M[idx, idx]
  return((determinant(A, log = TRUE)$modulus))} 

ObjFunMin <- function(x){
  ids <- order(x)
  idx <- ids[1:k]
  A <- M[idx, idx]
  return(-(determinant(A, log = TRUE)$modulus))} 

ObjFunMinCE <- function(x){
  ids <- order(x[1:n])
  idx <- ids[1:k]
  A <- M[idx,idx ]
  return(-(determinant(A, log = TRUE)$modulus))} 

library("kofnGA")
library("DEoptim")
library("pso")
library("GA")
library("CEoptim")


n <<- N
k <<- 15

ubnd = rep.int(1, n)
lbnd = rep.int(0, n)

repNum = 20

#############################################################
# kofnGA
#############################################################

ObjFun <- function(idx, Matrix) -(determinant(Matrix[idx, idx], log = TRUE)$modulus)
set.seed(12345)
for(i in 1:repNum){
  ptm <- proc.time()

  result <- kofnGA(n = 200, k = k, popsize = 1000,  
                   OF = ObjFun, keepbest = 100, ngen=40000,
                   tourneysize = 100, Matrix = M)
  t = proc.time() - ptm
  
  print(paste0(result$bestobj," ",t[3]))
  } 

#############################################################
# Differential Evolution
#############################################################
set.seed(12345)
print("DE")
print("----------------------------------------------------")
for(i in 1:repNum) {
  ptm <- proc.time()
  outDEoptim <- DEoptim(ObjFunMin, lbnd, ubnd,                         
                        DEoptim.control(NP = 10*n, itermax = 200,
                                        trace=FALSE,strategy=2))
  
  t = proc.time() - ptm
  print(paste0(outDEoptim$optim$bestval," ",t[3]))
}

#############################################################
# PSO
#############################################################
set.seed(12345)
print("PSO")
print("----------------------------------------------------")
for(i in 1:repNum) {
  ptm <- proc.time()
  
  outPSOoptim <- psoptim(rep(NA,n), ObjFunMin, 
                         lower = lbnd, upper = ubnd,
                         control=list(trace=0, s = 10*n, maxit=45))
  t = proc.time() - ptm
  print(paste0(outPSOoptim$value," ",t[3]))
}
print("----------------------------------------------------")
  

#############################################################
# GA
#############################################################
set.seed(12345)
print("GA")
print("----------------------------------------------------")
for(i in 1:repNum) {
  ptm <- proc.time()
 
  GA <- ga(type = "real-valued", fitness = ObjFunMax, 
           lower = lbnd, popSize = 6*n, maxiter = 20, monitor=1, upper = ubnd)
  
  t = proc.time() - ptm
  print(paste0(attr(GA, "fitnessValue")," ",t[3] ))
}

#############################################################
# CE
#############################################################

 
set.seed(12345)
mu0 <- rep(0,n); sigma0 <- rep(1,n)
print("CE")
print("----------------------------------------------------")
for(i in 1:repNum) {
  ptm <- proc.time()
res <- CEoptim(ObjFunMinCE,  
               continuous=list(mean=mu0, sd=sigma0,
                               smoothMean=0.7, smoothSd=0.7),
               maximize=FALSE, N=10*n, rho=0.1,
               iterThr=1e4L,noImproveThr=5, verbose=FALSE)
t = proc.time() - ptm
print(paste0(res$optimum," ",t[3] ))
}


